@@ -86,12 +86,11 @@ start_again:
   buf_block_t *trx_sys_block= buf_dblwr_trx_sys_get(&mtr);
 
   if (mach_read_from_4(TRX_SYS_DOUBLEWRITE + TRX_SYS_DOUBLEWRITE_MAGIC +
-                       trx_sys_block->page.frame) ==
-      TRX_SYS_DOUBLEWRITE_MAGIC_N)
+                       trx_sys_block->frame) == TRX_SYS_DOUBLEWRITE_MAGIC_N)
   {
     /* The doublewrite buffer has already been created: just read in
     some numbers */
-    init(TRX_SYS_DOUBLEWRITE + trx_sys_block->page.frame);
+    init(TRX_SYS_DOUBLEWRITE + trx_sys_block->frame);
     mtr.commit();
     return true;
   }
@@ -122,7 +121,7 @@ too_small:
   }
 
   byte *fseg_header= TRX_SYS_DOUBLEWRITE + TRX_SYS_DOUBLEWRITE_FSEG +
-    trx_sys_block->page.frame;
+    trx_sys_block->frame;
   for (uint32_t prev_page_no= 0, i= 0, extent_size= FSP_EXTENT_SIZE;
        i < 2 * size + extent_size / 2; i++)
   {
@@ -150,12 +149,12 @@ too_small:
     tablespace, then the page has not been written to in
     doublewrite. */
 
-    ut_ad(new_block->page.lock.not_recursive());
+    ut_ad(new_block->lock.not_recursive());
     const page_id_t id= new_block->page.id();
     /* We only do this in the debug build, to ensure that the check in
     buf_flush_init_for_writing() will see a valid page type. The
     flushes of new_block are actually unnecessary here.  */
-    ut_d(mtr.write<2>(*new_block, FIL_PAGE_TYPE + new_block->page.frame,
+    ut_d(mtr.write<2>(*new_block, FIL_PAGE_TYPE + new_block->frame,
                       FIL_PAGE_TYPE_SYS));
 
     if (i == size / 2)
@@ -163,10 +162,10 @@ too_small:
       ut_a(id.page_no() == size);
       mtr.write<4>(*trx_sys_block,
                    TRX_SYS_DOUBLEWRITE + TRX_SYS_DOUBLEWRITE_BLOCK1 +
-                   trx_sys_block->page.frame, id.page_no());
+                   trx_sys_block->frame, id.page_no());
       mtr.write<4>(*trx_sys_block,
                    TRX_SYS_DOUBLEWRITE + TRX_SYS_DOUBLEWRITE_REPEAT +
-                   TRX_SYS_DOUBLEWRITE_BLOCK1 + trx_sys_block->page.frame,
+                   TRX_SYS_DOUBLEWRITE_BLOCK1 + trx_sys_block->frame,
                    id.page_no());
     }
     else if (i == size / 2 + size)
@@ -174,10 +173,10 @@ too_small:
       ut_a(id.page_no() == 2 * size);
       mtr.write<4>(*trx_sys_block,
                    TRX_SYS_DOUBLEWRITE + TRX_SYS_DOUBLEWRITE_BLOCK2 +
-                   trx_sys_block->page.frame, id.page_no());
+                   trx_sys_block->frame, id.page_no());
       mtr.write<4>(*trx_sys_block,
                    TRX_SYS_DOUBLEWRITE + TRX_SYS_DOUBLEWRITE_REPEAT +
-                   TRX_SYS_DOUBLEWRITE_BLOCK2 + trx_sys_block->page.frame,
+                   TRX_SYS_DOUBLEWRITE_BLOCK2 + trx_sys_block->frame,
                    id.page_no());
     }
     else if (i > size / 2)
@@ -194,7 +193,7 @@ too_small:
       mtr.start();
       trx_sys_block= buf_dblwr_trx_sys_get(&mtr);
       fseg_header= TRX_SYS_DOUBLEWRITE + TRX_SYS_DOUBLEWRITE_FSEG +
-        trx_sys_block->page.frame;
+        trx_sys_block->frame;
     }
 
     prev_page_no= id.page_no();
@@ -202,19 +201,19 @@ too_small:
 
   mtr.write<4>(*trx_sys_block,
                TRX_SYS_DOUBLEWRITE + TRX_SYS_DOUBLEWRITE_MAGIC +
-               trx_sys_block->page.frame, TRX_SYS_DOUBLEWRITE_MAGIC_N);
+               trx_sys_block->frame, TRX_SYS_DOUBLEWRITE_MAGIC_N);
   mtr.write<4>(*trx_sys_block,
                TRX_SYS_DOUBLEWRITE + TRX_SYS_DOUBLEWRITE_MAGIC +
-               TRX_SYS_DOUBLEWRITE_REPEAT + trx_sys_block->page.frame,
+               TRX_SYS_DOUBLEWRITE_REPEAT + trx_sys_block->frame,
                TRX_SYS_DOUBLEWRITE_MAGIC_N);
 
   mtr.write<4>(*trx_sys_block,
                TRX_SYS_DOUBLEWRITE + TRX_SYS_DOUBLEWRITE_SPACE_ID_STORED +
-               trx_sys_block->page.frame,
-               TRX_SYS_DOUBLEWRITE_SPACE_ID_STORED_N);
+               trx_sys_block->frame, TRX_SYS_DOUBLEWRITE_SPACE_ID_STORED_N);
   mtr.commit();
 
-  buf_flush_wait_flushed(mtr.commit_lsn());
+  /* Flush the modified pages to disk and make a checkpoint */
+  log_make_checkpoint();
 
   /* Remove doublewrite pages from LRU */
   buf_pool_invalidate();
@@ -352,7 +351,7 @@ void buf_dblwr_t::recover()
     if (recv_sys.parse_start_lsn > lsn)
       /* Pages written before the checkpoint are not useful for recovery. */
       continue;
-    const ulint space_id= page_get_space_id(page);
+    const uint32_t space_id= page_get_space_id(page);
     const page_id_t page_id(space_id, page_no);
 
     if (recv_sys.scanned_lsn < lsn)
@@ -520,9 +519,8 @@ static void buf_dblwr_check_page_lsn(const buf_page_t &b, const byte *page)
 /** Check the LSN values on the page with which this block is associated. */
 static void buf_dblwr_check_block(const buf_page_t *bpage)
 {
-  ut_ad(bpage->in_file());
-  const page_t *page= bpage->frame;
-  ut_ad(page);
+  ut_ad(bpage->state() == BUF_BLOCK_FILE_PAGE);
+  const page_t *page= reinterpret_cast<const buf_block_t*>(bpage)->frame;
 
   switch (fil_page_get_type(page)) {
   case FIL_PAGE_INDEX:
@@ -596,8 +594,8 @@ bool buf_dblwr_t::flush_buffered_writes(const ulint size)
     ut_d(buf_dblwr_check_page_lsn(*bpage, write_buf + len2));
   }
 #endif /* UNIV_DEBUG */
-  const IORequest request{nullptr, nullptr, fil_system.sys_space->chain.start,
-                          IORequest::DBLWR_BATCH};
+  const IORequest request(nullptr, fil_system.sys_space->chain.start,
+                          IORequest::DBLWR_BATCH);
   ut_a(fil_system.sys_space->acquire());
   if (multi_batch)
   {
@@ -616,14 +614,6 @@ bool buf_dblwr_t::flush_buffered_writes(const ulint size)
   return true;
 }
 
-static void *get_frame(const IORequest &request)
-{
-  if (request.slot)
-    return request.slot->out_buf;
-  const buf_page_t *bpage= request.bpage;
-  return bpage->zip.data ? bpage->zip.data : bpage->frame;
-}
-
 void buf_dblwr_t::flush_buffered_writes_completed(const IORequest &request)
 {
   ut_ad(this == &buf_dblwr);
@@ -661,8 +651,9 @@ void buf_dblwr_t::flush_buffered_writes_completed(const IORequest &request)
     buf_page_t* bpage= e.request.bpage;
     ut_ad(bpage->in_file());
 
-    void *frame= get_frame(e.request);
-    ut_ad(frame);
+    /* We request frame here to get correct buffer in case of
+    encryption and/or page compression */
+    void *frame= buf_page_get_frame(bpage);
 
     auto e_size= e.size;
 
@@ -673,6 +664,7 @@ void buf_dblwr_t::flush_buffered_writes_completed(const IORequest &request)
     }
     else
     {
+      ut_ad(bpage->state() == BUF_BLOCK_FILE_PAGE);
       ut_ad(!bpage->zip_size());
       ut_d(buf_dblwr_check_page_lsn(*bpage, static_cast<const byte*>(frame)));
     }
@@ -740,9 +732,13 @@ void buf_dblwr_t::add_to_batch(const IORequest &request, size_t size)
 
   byte *p= active_slot->write_buf + srv_page_size * active_slot->first_free;
 
+  /* We request frame here to get correct buffer in case of
+  encryption and/or page compression */
+  void *frame= buf_page_get_frame(request.bpage);
+
   /* "frame" is at least 1024-byte aligned for ROW_FORMAT=COMPRESSED pages,
   and at least srv_page_size (4096-byte) for everything else. */
-  memcpy_aligned<UNIV_ZIP_SIZE_MIN>(p, get_frame(request), size);
+  memcpy_aligned<UNIV_ZIP_SIZE_MIN>(p, frame, size);
   /* fil_page_compress() for page_compressed guarantees 256-byte alignment */
   memset_aligned<256>(p + size, 0, srv_page_size - size);
   /* FIXME: Inform the compiler that "size" and "srv_page_size - size"
