@@ -1,7 +1,7 @@
 /*****************************************************************************
 
 Copyright (c) 1995, 2017, Oracle and/or its affiliates. All Rights Reserved.
-Copyright (c) 2017, 2022, MariaDB Corporation.
+Copyright (c) 2017, 2021, MariaDB Corporation.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -32,7 +32,6 @@ Created 11/26/1995 Heikki Tuuri
 #include "page0types.h"
 #include "mtr0log.h"
 #include "log0recv.h"
-#include "my_cpu.h"
 #ifdef BTR_CUR_HASH_ADAPT
 # include "btr0sea.h"
 #endif
@@ -167,18 +166,19 @@ struct FindPage
 			return(true);
 		}
 
-		buf_page_t* bpage = static_cast<buf_page_t*>(slot->object);
+		buf_block_t* block = reinterpret_cast<buf_block_t*>(
+			slot->object);
 
-		if (m_ptr < bpage->frame
-		    || m_ptr >= bpage->frame + srv_page_size) {
+		if (m_ptr < block->frame
+		    || m_ptr >= block->frame + srv_page_size) {
 			return(true);
 		}
 		ut_ad(!(slot->type & MTR_MEMO_PAGE_S_FIX)
-		      || bpage->lock.have_s());
+		      || block->lock.have_s());
 		ut_ad(!(slot->type & MTR_MEMO_PAGE_SX_FIX)
-		      || bpage->lock.have_u_or_x());
+		      || block->lock.have_u_or_x());
 		ut_ad(!(slot->type & MTR_MEMO_PAGE_X_FIX)
-		      || bpage->lock.have_x());
+		      || block->lock.have_x());
 		m_slot = slot;
 		return(false);
 	}
@@ -207,40 +207,41 @@ private:
 @param slot	memo slot */
 static void memo_slot_release(mtr_memo_slot_t *slot)
 {
-  void *object= slot->object;
-  slot->object= nullptr;
   switch (const auto type= slot->type) {
   case MTR_MEMO_S_LOCK:
-    static_cast<index_lock*>(object)->s_unlock();
+    static_cast<index_lock*>(slot->object)->s_unlock();
     break;
   case MTR_MEMO_X_LOCK:
   case MTR_MEMO_SX_LOCK:
-    static_cast<index_lock*>(object)->
+    static_cast<index_lock*>(slot->object)->
       u_or_x_unlock(type == MTR_MEMO_SX_LOCK);
     break;
   case MTR_MEMO_SPACE_X_LOCK:
-    static_cast<fil_space_t*>(object)->set_committed_size();
-    static_cast<fil_space_t*>(object)->x_unlock();
+    static_cast<fil_space_t*>(slot->object)->set_committed_size();
+    static_cast<fil_space_t*>(slot->object)->x_unlock();
     break;
   case MTR_MEMO_SPACE_S_LOCK:
-    static_cast<fil_space_t*>(object)->s_unlock();
+    static_cast<fil_space_t*>(slot->object)->s_unlock();
     break;
   default:
-    buf_page_t *bpage= static_cast<buf_page_t*>(object);
-    bpage->unfix();
-    switch (auto latch= slot->type & ~MTR_MEMO_MODIFY) {
+#ifdef UNIV_DEBUG
+    switch (slot->type & ~MTR_MEMO_MODIFY) {
+    case MTR_MEMO_BUF_FIX:
     case MTR_MEMO_PAGE_S_FIX:
-      bpage->lock.s_unlock();
-      return;
     case MTR_MEMO_PAGE_SX_FIX:
     case MTR_MEMO_PAGE_X_FIX:
-      bpage->lock.u_or_x_unlock(latch == MTR_MEMO_PAGE_SX_FIX);
-      /* fall through */
-    case MTR_MEMO_BUF_FIX:
-      return;
+      break;
+    default:
+      ut_ad("invalid type" == 0);
+      break;
     }
-    ut_ad("invalid type" == 0);
+#endif /* UNIV_DEBUG */
+    buf_block_t *block= static_cast<buf_block_t*>(slot->object);
+    buf_page_release_latch(block, slot->type & ~MTR_MEMO_MODIFY);
+    block->unfix();
+    break;
   }
+  slot->object= nullptr;
 }
 
 /** Release the latches acquired by the mini-transaction. */
@@ -248,42 +249,43 @@ struct ReleaseLatches {
   /** @return true always. */
   bool operator()(mtr_memo_slot_t *slot) const
   {
-    void *object= slot->object;
-    if (!object)
+    if (!slot->object)
       return true;
-    slot->object= nullptr;
     switch (const auto type= slot->type) {
     case MTR_MEMO_S_LOCK:
-      static_cast<index_lock*>(object)->s_unlock();
+      static_cast<index_lock*>(slot->object)->s_unlock();
       break;
     case MTR_MEMO_SPACE_X_LOCK:
-      static_cast<fil_space_t*>(object)->set_committed_size();
-      static_cast<fil_space_t*>(object)->x_unlock();
+      static_cast<fil_space_t*>(slot->object)->set_committed_size();
+      static_cast<fil_space_t*>(slot->object)->x_unlock();
       break;
     case MTR_MEMO_SPACE_S_LOCK:
-      static_cast<fil_space_t*>(object)->s_unlock();
+      static_cast<fil_space_t*>(slot->object)->s_unlock();
       break;
     case MTR_MEMO_X_LOCK:
     case MTR_MEMO_SX_LOCK:
-      static_cast<index_lock*>(object)->
+      static_cast<index_lock*>(slot->object)->
         u_or_x_unlock(type == MTR_MEMO_SX_LOCK);
       break;
     default:
-      buf_page_t *bpage= static_cast<buf_page_t*>(object);
-      bpage->unfix();
-      switch (auto latch= slot->type & ~MTR_MEMO_MODIFY) {
+#ifdef UNIV_DEBUG
+      switch (slot->type & ~MTR_MEMO_MODIFY) {
+      case MTR_MEMO_BUF_FIX:
       case MTR_MEMO_PAGE_S_FIX:
-        bpage->lock.s_unlock();
-        return true;
       case MTR_MEMO_PAGE_SX_FIX:
       case MTR_MEMO_PAGE_X_FIX:
-        bpage->lock.u_or_x_unlock(latch == MTR_MEMO_PAGE_SX_FIX);
-        /* fall through */
-      case MTR_MEMO_BUF_FIX:
-        return true;
+        break;
+      default:
+        ut_ad("invalid type" == 0);
+        break;
       }
-      ut_ad("invalid type" == 0);
+#endif /* UNIV_DEBUG */
+      buf_block_t *block= static_cast<buf_block_t*>(slot->object);
+      buf_page_release_latch(block, slot->type & ~MTR_MEMO_MODIFY);
+      block->unfix();
+      break;
     }
+    slot->object= NULL;
     return true;
   }
 };
@@ -299,50 +301,6 @@ struct ReleaseAll {
   }
 };
 
-/** Stops iteration is savepoint is reached */
-template <typename Functor> struct TillSavepoint
-{
-
-  /** Constructor
-  @param[in] functor functor which is called if savepoint is not reached
-  @param[in] savepoint savepoint value to rollback
-  @param[in] used current position in slots container */
-  TillSavepoint(const Functor &functor, ulint savepoint, ulint used)
-      : functor(functor),
-        m_slots_count((used - savepoint) / sizeof(mtr_memo_slot_t))
-  {
-    ut_ad(savepoint);
-    ut_ad(used >= savepoint);
-  }
-
-  /** @return true if savepoint is not reached, false otherwise */
-  bool operator()(mtr_memo_slot_t *slot)
-  {
-#ifdef UNIV_DEBUG
-    /** This check is added because the code is invoked only from
-    row_search_mvcc() to release latches acquired during clustered index search
-    for secondary index record. To make it more universal we could add one more
-    member in this functor for debug build to pass only certain slot types,
-    but this is currently not necessary. */
-    switch (slot->type)
-    {
-    case MTR_MEMO_S_LOCK:
-    case MTR_MEMO_PAGE_S_FIX:
-      break;
-    default:
-      ut_a(false);
-    }
-#endif
-    return m_slots_count-- && functor(slot);
-  }
-
-private:
-  /** functor to invoke */
-  const Functor &functor;
-  /** slots count left till savepoint */
-  ulint m_slots_count;
-};
-
 #ifdef UNIV_DEBUG
 /** Check that all slots have been handled. */
 struct DebugCheck {
@@ -448,27 +406,18 @@ void mtr_t::commit()
 
     std::pair<lsn_t,page_flush_ahead> lsns;
 
-    if (UNIV_LIKELY(m_log_mode == MTR_LOG_ALL))
-    {
-      lsns= do_write();
-
-      if (m_made_dirty)
-        mysql_mutex_lock(&log_sys.flush_order_mutex);
-
-      /* It is now safe to release log_sys.mutex because the
-      buf_pool.flush_order_mutex will ensure that we are the first one
-      to insert into buf_pool.flush_list. */
-      mysql_mutex_unlock(&log_sys.mutex);
-    }
+    if (const ulint len= prepare_write())
+      lsns= finish_write(len);
     else
-    {
-      ut_ad(m_log_mode == MTR_LOG_NO_REDO);
-      ut_ad(m_log.size() == 0);
-      m_commit_lsn= log_sys.get_lsn();
       lsns= { m_commit_lsn, PAGE_FLUSH_NO };
-      if (UNIV_UNLIKELY(m_made_dirty)) /* This should be IMPORT TABLESPACE */
-        mysql_mutex_lock(&log_sys.flush_order_mutex);
-    }
+
+    if (m_made_dirty)
+      mysql_mutex_lock(&log_sys.flush_order_mutex);
+
+    /* It is now safe to release the log mutex because the
+    flush_order mutex will ensure that we are the first one
+    to insert into the flush list. */
+    mysql_mutex_unlock(&log_sys.mutex);
 
     if (m_freed_pages)
     {
@@ -512,21 +461,6 @@ void mtr_t::commit()
   release_resources();
 }
 
-/** Release latches till savepoint. To simplify the code only
-MTR_MEMO_S_LOCK and MTR_MEMO_PAGE_S_FIX slot types are allowed to be
-released, otherwise it would be neccesary to add one more argument in the
-function to point out what slot types are allowed for rollback, and this
-would be overengineering as corrently the function is used only in one place
-in the code.
-@param savepoint   savepoint, can be obtained with get_savepoint */
-void mtr_t::rollback_to_savepoint(ulint savepoint)
-{
-  Iterate<TillSavepoint<ReleaseLatches>> iteration(
-      TillSavepoint<ReleaseLatches>(ReleaseLatches(), savepoint,
-                                    get_savepoint()));
-  m_memo.for_each_block_in_reverse(iteration);
-}
-
 /** Shrink a tablespace. */
 struct Shrink
 {
@@ -551,11 +485,8 @@ struct Shrink
     case MTR_MEMO_PAGE_X_FIX:
     case MTR_MEMO_PAGE_SX_FIX:
       auto &bpage= static_cast<buf_block_t*>(slot->object)->page;
-      const auto s= bpage.state();
-      ut_ad(s >= buf_page_t::FREED);
-      ut_ad(s < buf_page_t::READ_FIX);
-      ut_ad(bpage.frame);
-      const page_id_t id{bpage.id()};
+      ut_ad(bpage.io_fix() == BUF_IO_NONE);
+      const auto id= bpage.id();
       if (id < high)
       {
         ut_ad(id.space() == high.space() ||
@@ -563,11 +494,10 @@ struct Shrink
                srv_is_undo_tablespace(high.space())));
         break;
       }
-      if (s >= buf_page_t::UNFIXED)
-        bpage.set_freed(s);
       ut_ad(id.space() == high.space());
+      ut_ad(bpage.state() == BUF_BLOCK_FILE_PAGE);
       if (bpage.oldest_modification() > 1)
-        bpage.reset_oldest_modification();
+        bpage.clear_oldest_modification(false);
       slot->type= static_cast<mtr_memo_type_t>(slot->type & ~MTR_MEMO_MODIFY);
     }
     return true;
@@ -589,7 +519,7 @@ void mtr_t::commit_shrink(fil_space_t &space)
 
   log_write_and_flush_prepare();
 
-  const lsn_t start_lsn= do_write().first;
+  const lsn_t start_lsn= finish_write(prepare_write()).first;
 
   mysql_mutex_lock(&log_sys.flush_order_mutex);
   /* Durably write the reduced FSP_SIZE before truncating the data file. */
@@ -685,7 +615,7 @@ void mtr_t::commit_files(lsn_t checkpoint_lsn)
 @param[in]	space	tablespace
 @return whether the mini-transaction is associated with the space */
 bool
-mtr_t::is_named_space(ulint space) const
+mtr_t::is_named_space(uint32_t space) const
 {
 	ut_ad(!m_user_space || m_user_space->id != TRX_SYS_SPACE);
 
@@ -725,8 +655,7 @@ bool mtr_t::is_named_space(const fil_space_t* space) const
 /** Acquire a tablespace X-latch.
 @param[in]	space_id	tablespace ID
 @return the tablespace object (never NULL) */
-fil_space_t*
-mtr_t::x_lock_space(ulint space_id)
+fil_space_t *mtr_t::x_lock_space(uint32_t space_id)
 {
 	fil_space_t*	space;
 
@@ -1004,10 +933,19 @@ struct mtr_write_log
   }
 };
 
-std::pair<lsn_t,mtr_t::page_flush_ahead> mtr_t::do_write()
+/** Prepare to write the mini-transaction log to the redo log buffer.
+@return number of bytes to write in finish_write() */
+inline ulint mtr_t::prepare_write()
 {
 	ut_ad(!recv_no_log_write);
-	ut_ad(m_log_mode == MTR_LOG_ALL);
+
+	if (UNIV_UNLIKELY(m_log_mode != MTR_LOG_ALL)) {
+		ut_ad(m_log_mode == MTR_LOG_NO_REDO);
+		ut_ad(m_log.size() == 0);
+		mysql_mutex_lock(&log_sys.mutex);
+		m_commit_lsn = log_sys.get_lsn();
+		return 0;
+	}
 
 	ulint	len	= m_log.size();
 	ut_ad(len > 0);
@@ -1039,7 +977,7 @@ std::pair<lsn_t,mtr_t::page_flush_ahead> mtr_t::do_write()
 	/* check and attempt a checkpoint if exceeding capacity */
 	log_margin_checkpoint_age(len);
 
-	return finish_write(len);
+	return(len);
 }
 
 /** Append the redo log records to the redo log buffer.
@@ -1118,7 +1056,7 @@ bool mtr_t::have_x_latch(const buf_block_t &block) const
                                  MTR_MEMO_BUF_FIX | MTR_MEMO_MODIFY));
     return false;
   }
-  ut_ad(block.page.lock.have_x());
+  ut_ad(block.lock.have_x());
   return true;
 }
 
@@ -1153,21 +1091,20 @@ static void mtr_defer_drop_ahi(buf_block_t *block, mtr_memo_type_t fix_type)
     break;
   case MTR_MEMO_PAGE_S_FIX:
     /* Temporarily release our S-latch. */
-    block->page.lock.s_unlock();
-    block->page.lock.x_lock();
+    block->lock.s_unlock();
+    block->lock.x_lock();
     if (dict_index_t *index= block->index)
       if (index->freed())
         btr_search_drop_page_hash_index(block);
-    block->page.lock.x_unlock();
-    block->page.lock.s_lock();
-    ut_ad(!block->page.is_read_fixed());
+    block->lock.x_unlock();
+    block->lock.s_lock();
     break;
   case MTR_MEMO_PAGE_SX_FIX:
-    block->page.lock.u_x_upgrade();
+    block->lock.u_x_upgrade();
     if (dict_index_t *index= block->index)
       if (index->freed())
         btr_search_drop_page_hash_index(block);
-    block->page.lock.x_u_downgrade();
+    block->lock.x_u_downgrade();
     break;
   default:
     ut_ad(fix_type == MTR_MEMO_PAGE_X_FIX);
@@ -1193,7 +1130,7 @@ struct UpgradeX
 /** Upgrade U locks on a block to X */
 void mtr_t::page_lock_upgrade(const buf_block_t &block)
 {
-  ut_ad(block.page.lock.have_x());
+  ut_ad(block.lock.have_x());
   m_memo.for_each_block(CIterate<UpgradeX>((UpgradeX(block))));
 #ifdef BTR_CUR_HASH_ADAPT
   ut_ad(!block.index || !block.index->freed());
@@ -1227,9 +1164,6 @@ void mtr_t::lock_upgrade(const index_lock &lock)
 void mtr_t::page_lock(buf_block_t *block, ulint rw_latch)
 {
   mtr_memo_type_t fix_type;
-  ut_d(const auto state= block->page.state());
-  ut_ad(state > buf_page_t::FREED);
-  ut_ad(state > buf_page_t::WRITE_FIX || state < buf_page_t::READ_FIX);
   switch (rw_latch)
   {
   case RW_NO_LATCH:
@@ -1237,23 +1171,21 @@ void mtr_t::page_lock(buf_block_t *block, ulint rw_latch)
     goto done;
   case RW_S_LATCH:
     fix_type= MTR_MEMO_PAGE_S_FIX;
-    block->page.lock.s_lock();
+    block->lock.s_lock();
     break;
   case RW_SX_LATCH:
     fix_type= MTR_MEMO_PAGE_SX_FIX;
-    block->page.lock.u_lock();
-    ut_ad(!block->page.is_io_fixed());
+    block->lock.u_lock();
     break;
   default:
     ut_ad(rw_latch == RW_X_LATCH);
     fix_type= MTR_MEMO_PAGE_X_FIX;
-    if (block->page.lock.x_lock_upgraded())
+    if (block->lock.x_lock_upgraded())
     {
-      block->unfix();
       page_lock_upgrade(*block);
+      block->unfix();
       return;
     }
-    ut_ad(!block->page.is_io_fixed());
   }
 
 #ifdef BTR_CUR_HASH_ADAPT
@@ -1263,9 +1195,6 @@ void mtr_t::page_lock(buf_block_t *block, ulint rw_latch)
 #endif /* BTR_CUR_HASH_ADAPT */
 
 done:
-  ut_ad(state < buf_page_t::UNFIXED ||
-        page_id_t(page_get_space_id(block->page.frame),
-                  page_get_page_no(block->page.frame)) == block->page.id());
   memo_push(block, fix_type);
 }
 
@@ -1345,7 +1274,7 @@ struct FlaggedCheck {
 		if (f & (MTR_MEMO_PAGE_S_FIX | MTR_MEMO_PAGE_SX_FIX
 			 | MTR_MEMO_PAGE_X_FIX)) {
 			block_lock* lock = &static_cast<buf_block_t*>(
-				const_cast<void*>(m_ptr))->page.lock;
+				const_cast<void*>(m_ptr))->lock;
 			ut_ad(!(f & MTR_MEMO_PAGE_S_FIX) || lock->have_s());
 			ut_ad(!(f & MTR_MEMO_PAGE_SX_FIX)
 			      || lock->have_u_or_x());
@@ -1432,6 +1361,7 @@ void mtr_t::modify(const buf_block_t &block)
   {
     /* This must be PageConverter::update_page() in IMPORT TABLESPACE. */
     ut_ad(!block.page.in_LRU_list);
+    ut_ad(!buf_pool.is_uncompressed(&block));
     return;
   }
 
