@@ -807,13 +807,10 @@ static json_state_handler json_actions[NR_JSON_STATES][NR_C_CLASSES]=
 int json_scan_start(json_engine_t *je,
                     CHARSET_INFO *i_cs, const uchar *str, const uchar *end)
 {
-  static const uchar no_time_to_die= 0;
-
   json_string_setup(&je->s, i_cs, str, end);
   je->stack[0]= JST_DONE;
   je->stack_p= 0;
   je->state= JST_VALUE;
-  je->killed_ptr = (uchar*)&no_time_to_die;
   return 0;
 }
 
@@ -951,7 +948,7 @@ int json_read_value(json_engine_t *j)
 {
   int t_next, c_len, res;
 
-  j->value_type= JSON_VALUE_UNINITALIZED;
+  j->value_type= JSON_VALUE_UNINITIALIZED;
   if (j->state == JST_KEY)
   {
     while (json_read_keyname_chr(j) == 0) {}
@@ -974,7 +971,7 @@ int json_scan_next(json_engine_t *j)
   int t_next;
 
   get_first_nonspace(&j->s, &t_next, &j->sav_c_len);
-  return *j->killed_ptr || json_actions[j->state][t_next](j);
+  return json_actions[j->state][t_next](j);
 }
 
 
